#include <assert.h>
#include <math.h>
#include <FL/Fl.H>
#include <FL/Fl_Image.H>
#include "features.h"
#include "ImageLib/FileIO.h"

#define PI 3.14159265358979323846
#define MULTI_FACTOR 100
#define THRESHOLD 0.001

#include <vector>
#include <algorithm>
using namespace std;

// Compute features of an image.
bool computeFeatures(CFloatImage &image, FeatureSet &features, int featureType) {
	// TODO: Instead of calling dummyComputeFeatures, write your own
	// feature computation routines and call them here.
	switch (featureType) {
	case 1:
		dummyComputeFeatures(image, features);
		break;
	case 2:
		ComputeHarrisFeatures(image, features);
		break;
	case 3:
		ComputeSimpleFeatures(image, features);
		break;
	default:
		return false;
	}

	// This is just to make sure the IDs are assigned in order, because
	// the ID gets used to index into the feature array.
	for (unsigned int i=0; i<features.size(); i++) {
		features[i].id = i+1;
	}

	return true;
}

// Perform a query on the database.  This simply runs matchFeatures on
// each image in the database, and returns the feature set of the best
// matching image.
bool performQuery(const FeatureSet &f, const ImageDatabase &db, int &bestIndex, vector<FeatureMatch> &bestMatches, double &bestScore, int matchType) {
	// Here's a nice low number.
	bestScore = -1e100;

	vector<FeatureMatch> tempMatches;
	double tempScore;

	for (unsigned int i=0; i<db.size(); i++) {
		if (!matchFeatures(f, db[i].features, tempMatches, tempScore, matchType)) {
			return false;
		}

		if (tempScore > bestScore) {
			bestIndex = i;
			bestScore = tempScore;
			bestMatches = tempMatches;
		}
	}

	return true;
}

// Match one feature set with another.
bool matchFeatures(const FeatureSet &f1, const FeatureSet &f2, vector<FeatureMatch> &matches, double &totalScore, int matchType) {
	// TODO: We have given you the ssd matching function, you must write your own
	// feature matching function for the ratio test.
	
	printf("\nMatching features.......\n");

	switch (matchType) {
	case 1:
		ssdMatchFeatures(f1, f2, matches, totalScore);
		return true;
	case 2:
		ratioMatchFeatures(f1, f2, matches, totalScore);
		return true;
	default:
		return false;
	}
}

// Evaluate a match using a ground truth homography.  This computes the
// average SSD distance between the matched feature points and
// the actual transformed positions.
double evaluateMatch(const FeatureSet &f1, const FeatureSet &f2, const vector<FeatureMatch> &matches, double h[9]) {
	double d = 0;
	int n = 0;

	double xNew;
	double yNew;

    unsigned int num_matches = matches.size();
	for (unsigned int i=0; i<num_matches; i++) {
		int id1 = matches[i].id1;
        int id2 = matches[i].id2;
        applyHomography(f1[id1-1].x, f1[id1-1].y, xNew, yNew, h);
		d += sqrt(pow(xNew-f2[id2-1].x,2)+pow(yNew-f2[id2-1].y,2));
		n++;
	}	

	return d / n;
}

void addRocData(const FeatureSet &f1, const FeatureSet &f2, const vector<FeatureMatch> &matches, double h[9],vector<bool> &isMatch,double threshold,double &maxD) {
	double d = 0;

	double xNew;
	double yNew;

    unsigned int num_matches = matches.size();
	for (unsigned int i=0; i<num_matches; i++) {
		int id1 = matches[i].id1;
        int id2 = matches[i].id2;
		applyHomography(f1[id1-1].x, f1[id1-1].y, xNew, yNew, h);

		// Ignore unmatched points.  There might be a better way to
		// handle this.
		d = sqrt(pow(xNew-f2[id2-1].x,2)+pow(yNew-f2[id2-1].y,2));
		if (d<=threshold)
		{
			isMatch.push_back(1);
		}
		else
		{
			isMatch.push_back(0);
		}

		if (matches[i].score>maxD)
			maxD=matches[i].score;
	}	
}

vector<ROCPoint> computeRocCurve(vector<FeatureMatch> &matches,vector<bool> &isMatch,vector<double> &thresholds)
{
	vector<ROCPoint> dataPoints;

	for (int i=0; i < (int)thresholds.size();i++)
	{
		//printf("Checking threshold: %lf.\r\n",thresholds[i]);
		int tp=0;
		int actualCorrect=0;
		int fp=0;
		int actualError=0;
		int total=0;

        int num_matches = (int) matches.size();
		for (int j=0;j < num_matches;j++)
		{
			if (isMatch[j])
			{
				actualCorrect++;
				if (matches[j].score<thresholds[i])
				{
					tp++;
				}
			}
			else
			{
				actualError++;
				if (matches[j].score<thresholds[i])
				{
					fp++;
				}
            }				
			
			total++;
		}

		ROCPoint newPoint;
		//printf("newPoints: %lf,%lf",newPoint.trueRate,newPoint.falseRate);
		newPoint.trueRate=(double(tp)/actualCorrect);
		newPoint.falseRate=(double(fp)/actualError);
		//printf("newPoints: %lf,%lf",newPoint.trueRate,newPoint.falseRate);

		dataPoints.push_back(newPoint);
	}

	return dataPoints;
}


// Compute silly example features.  This doesn't do anything
// meaningful.
void dummyComputeFeatures(CFloatImage &image, FeatureSet &features) {
	CShape sh = image.Shape();
	Feature f;

	for (int y=0; y<sh.height; y++) {
		for (int x=0; x<sh.width; x++) {
			double r = image.Pixel(x,y,0);
			double g = image.Pixel(x,y,1);
			double b = image.Pixel(x,y,2);

			if ((int)(255*(r+g+b)+0.5) % 100  == 1) {
				// If the pixel satisfies this meaningless criterion,
				// make it a feature.
				
				f.type = 1;
				f.id += 1;
				f.x = x;
				f.y = y;

				f.data.resize(1);
				f.data[0] = r + g + b;

				features.push_back(f);
			}
		}
	}
}

void ComputeGradient(CFloatImage &srcImage, CFloatImage &dxImage, CFloatImage &dyImage) 
{
	int w = srcImage.Shape().width;
    int h = srcImage.Shape().height;
	int sigma = 3;
	
	CFloatImage smoothedImage(w, h, 1);

	for (int y = 0; y < h; y++) {
        for (int x = 0; x < w; x++) {
			double pixel_value = 0.0;
			for (int y_offset = -sigma / 2; y_offset <= sigma / 2; y_offset++) {
				for (int x_offset = -sigma / 2; x_offset <= sigma / 2; x_offset++) {
					// double coeff = 1.0 / (sqrt(2 * PI) * 1.0) * exp(-(y_offset * y_offset + x_offset * x_offset) / (2.0 * 1.0 * 1.0));
					double coeff = 1.0 / (sigma * sigma);
					int y_pos, x_pos;
					y_pos = max(y + y_offset, 0);
					y_pos = min(y_pos, h - 1);
					x_pos = max(x + x_offset, 0);
					x_pos = min(x_pos, w - 1);

					pixel_value += coeff * srcImage.Pixel(x_pos, y_pos, 0);
				}
			}
			// printf( "I(%d,%d) = %lf -> %lf\n", x, y, srcImage.Pixel(x, y, 0), pixel_value);
			smoothedImage.Pixel(x, y, 0) = pixel_value;
        }
    }

	vector <double> coeff_dx(sigma * sigma);
	vector <double> coeff_dy(sigma * sigma);
	int idx = 0;
	double v1 = - (int)sigma / 2, v2 = -(int)sigma / 2;

	for (int y_offset = -sigma / 2; y_offset <= sigma / 2; y_offset++) {
		v2 = -(int)sigma / 2;
		for (int x_offset = -sigma / 2; x_offset <= sigma / 2; x_offset++) {
			coeff_dy[idx] = v1;
			coeff_dx[idx] = v2;
			idx++;
			v2 += 1;
		}
		v1 += 1;
	}
	

	for (int y = 0; y < h; y++) {
        for (int x = 0; x < w; x++) {
			double dx = 0.0, dy = 0.0;
			int idx = 0;
			for (int y_offset = -sigma / 2; y_offset <= sigma / 2; y_offset++) {
				for (int x_offset = -sigma / 2; x_offset <= sigma / 2; x_offset++) {
					int y_pos, x_pos;
					y_pos = max(y + y_offset, 0);
					y_pos = min(y_pos, h - 1);
					x_pos = max(x + x_offset, 0);
					x_pos = min(x_pos, w - 1);

					dx += coeff_dx[idx] * smoothedImage.Pixel(x_pos, y_pos, 0);
					dy += coeff_dy[idx] * smoothedImage.Pixel(x_pos, y_pos, 0);

					idx++;
				}
			}
			dxImage.Pixel(x, y, 0) = dx;
			dyImage.Pixel(x, y, 0) = dy;
        }
    }
}

void ComputeHarrisFeatures(CFloatImage &image, FeatureSet &features)
{
	//Create grayscale image used for Harris detection
	CFloatImage grayImage=ConvertToGray(image);

	//Create image to store Harris values
	CFloatImage harrisImage(image.Shape().width,image.Shape().height,1);

	//Create image to store local maximum harris values as 1, other pixels 0
	CByteImage harrisMaxImage(image.Shape().width,image.Shape().height,1);

	
	//compute Harris values puts harris values at each pixel position in harrisImage. 
	//You'll need to implement this function.
    computeHarrisValues(grayImage, harrisImage);
	
	// Threshold the harris image and compute local maxima.  You'll need to implement this function.
	computeLocalMaxima(harrisImage,harrisMaxImage);

    
    // Prints out the harris image for debugging purposes
	CByteImage tmp(harrisImage.Shape());
	convertToByteImage(harrisImage, tmp);
    WriteFile(tmp, "harris.tga");
    

	// TO DO--------------------------------------------------------------------
	//Loop through feature points in harrisMaxImage and create feature descriptor 
	//for each point above a threshold

    for (int y=0;y<harrisMaxImage.Shape().height;y++) {
		for (int x=0;x<harrisMaxImage.Shape().width;x++) {
		

			// Skip over non-maxima
            if (harrisMaxImage.Pixel(x, y, 0) == 0)
                continue;

			// printf("Harris Value(%d, %d) = (%d, %lf)\n", x, y, harrisMaxImage.Pixel(x, y, 0), harrisImage.Pixel(x, y, 0));

			// MODIFY: thresholding
			if (harrisImage.Pixel(x, y, 0) < THRESHOLD * MULTI_FACTOR)
				continue;
		    

            //TO DO---------------------------------------------------------------------
		    // Fill in feature with descriptor data here. 
            Feature f;

			// MODIFY: fill descriptor
			f.type = 2;
			f.x = x;
			f.y = y;
			

			// 8 orientation histograms for (1 + 2 * 2) grids with 8 x 8 pixels
			f.data = vector<double> (40);

			int patch_size = 16;
			CFloatImage patch = image.SubImage(x - patch_size / 2, y - patch_size / 2, patch_size, patch_size);
			CFloatImage dxImage(patch_size, patch_size, 1);
			CFloatImage dyImage(patch_size, patch_size, 1);
			ComputeGradient(patch, dxImage, dyImage);

			int sy[5] = {0, 0, 0, patch_size / 2, patch_size / 2}, ey[5] = {patch_size, patch_size / 2, patch_size / 2, patch_size, patch_size};
			int sx[5] = {0, 0, patch_size / 2, 0, patch_size / 2}, ex[5] = {patch_size, patch_size / 2, patch_size, patch_size / 2, patch_size};
			double grad_hist[5][8] = {0.0, };
			
			for (int grid_idx = 0; grid_idx < 5; grid_idx++) {
				for (int patch_y = sy[grid_idx]; patch_y < ey[grid_idx]; patch_y++) {
					for (int patch_x = sx[grid_idx]; patch_x < ex[grid_idx]; patch_x++) {
						double dx = dxImage.Pixel(patch_x, patch_y, 0);						
						double dy = dyImage.Pixel(patch_x, patch_y, 0);
						if (dx == 0.0 && dy == 0.0)
							continue;
						double grad_angle = (atan2(dy, dx) / PI + 1.0) * 8; // grad_angle = 0 ~ 8
						grad_angle = max(grad_angle, 0.0);
						grad_angle = min(grad_angle, 7.9);
						
						grad_hist[grid_idx][(int)grad_angle] += sqrt(dx * dx + dy * dy);
					}
				}
			}

			int max_idx = 0;
			double max_value = -1;
			for (int ori_idx = 0; ori_idx < 8; ori_idx++) {
				if (max_value < grad_hist[0][ori_idx]) {
					max_value = grad_hist[0][ori_idx];
					max_idx = ori_idx;
				}
				//grad_hist[0][ori_idx] /= 2.0;
			}

			int idx = 0;
			for (int grid_idx = 0; grid_idx < 5; grid_idx++) {
				for (int ori_idx = max_idx; ori_idx < 8; ori_idx++) {
					f.data[idx] = grad_hist[grid_idx][ori_idx];
					idx++;
				}
				for (int ori_idx = 0; ori_idx < max_idx; ori_idx++) {
					f.data[idx] = grad_hist[grid_idx][ori_idx];
					idx++;
				}
			}

			f.angleRadians = max_idx / 8 * PI;

            // Add the feature to the list of features
            features.push_back(f);
        }
	}
}


void ComputeSimpleFeatures(CFloatImage &image, FeatureSet &features)
{
	//Create grayscale image used for Harris detection
	CFloatImage grayImage=ConvertToGray(image);

	//Create image to store Harris values
	CFloatImage harrisImage(image.Shape().width,image.Shape().height,1);

	//Create image to store local maximum harris values as 1, other pixels 0
	CByteImage harrisMaxImage(image.Shape().width,image.Shape().height,1);

	
	//compute Harris values puts harris values at each pixel position in harrisImage. 
	//You'll need to implement this function.
    computeHarrisValues(grayImage, harrisImage);
	
	// Threshold the harris image and compute local maxima.  You'll need to implement this function.
	computeLocalMaxima(harrisImage,harrisMaxImage);

    
    // Prints out the harris image for debugging purposes
	CByteImage tmp(harrisImage.Shape());
	convertToByteImage(harrisImage, tmp);
    WriteFile(tmp, "harris.tga");
    

	// TO DO--------------------------------------------------------------------
	//Loop through feature points in harrisMaxImage and create feature descriptor 
	//for each point above a threshold

    for (int y=0;y<harrisMaxImage.Shape().height;y++) {
		for (int x=0;x<harrisMaxImage.Shape().width;x++) {
		

			// Skip over non-maxima
            if (harrisMaxImage.Pixel(x, y, 0) == 0)
                continue;

			// printf("Harris Value(%d, %d) = (%d, %lf)\n", x, y, harrisMaxImage.Pixel(x, y, 0), harrisImage.Pixel(x, y, 0));

			// MODIFY: thresholding
			if (harrisImage.Pixel(x, y, 0) < THRESHOLD * MULTI_FACTOR)
				continue;
		    

            //TO DO---------------------------------------------------------------------
		    // Fill in feature with descriptor data here. 
            Feature f;

			// MODIFY: fill descriptor
			f.type = 2;
			f.x = x;
			f.y = y;
			f.angleRadians = 0;			

			// grids with 5 x 5 pixels
			f.data = vector<double> (25);
			for (int i = -2; i <= 2; i++) {
				for (int j = -2; j <= 2; j++) {
					f.data.push_back(harrisImage.Pixel(x + j, y + i, 0));
				}
			}

            // Add the feature to the list of features
            features.push_back(f);
        }
	}
}

//TO DO---------------------------------------------------------------------
//Loop through the image to compute the harris corner values as described in class
// srcImage:  grayscale of original image
// harrisImage:  populate the harris values per pixel in this image
void computeHarrisValues(CFloatImage &srcImage, CFloatImage &harrisImage)
{
	// TODO:  Compute the harris score for 'srcImage' at this pixel and store in 'harrisImage'.  See the project
    //   page for pointers on how to do this

	// MODIFY: compute harris score

	int w = srcImage.Shape().width;
    int h = srcImage.Shape().height;

	int sigma = 5;
	CFloatImage dxImage(w, h, 1);
	CFloatImage dyImage(w, h, 1);

	ComputeGradient(srcImage, dxImage, dyImage);

	double total_coeff = 0.0;
	double gaussian_sigma = 3.0;

	for (int y_offset = -sigma / 2; y_offset <= sigma / 2; y_offset++) {
		for (int x_offset = -sigma / 2; x_offset <= sigma / 2; x_offset++) {
			total_coeff += 1.0 / (sqrt(2 * PI) * gaussian_sigma) * exp(-(y_offset * y_offset + x_offset * x_offset) / (2.0 * gaussian_sigma));
		}
	}
	
	for (int y = 0; y < h; y++) {
        for (int x = 0; x < w; x++) {
			double harris_value[4] = {0.0, };
			for (int y_offset = -sigma / 2; y_offset <= sigma / 2; y_offset++) {
				for (int x_offset = -sigma / 2; x_offset <= sigma / 2; x_offset++) {
					double coeff = 1.0 / (sqrt(2 * PI) * gaussian_sigma) * exp(-(y_offset * y_offset + x_offset * x_offset) / (2.0 * gaussian_sigma)) / total_coeff;
					int y_pos, x_pos;
					y_pos = max(y + y_offset, 0);
					y_pos = min(y_pos, h - 1);
					x_pos = max(x + x_offset, 0);
					x_pos = min(x_pos, w - 1);

					harris_value[0] += (coeff * dxImage.Pixel(x_pos, y_pos, 0) * dxImage.Pixel(x_pos, y_pos, 0));
					harris_value[1] += (coeff * dxImage.Pixel(x_pos, y_pos, 0) * dyImage.Pixel(x_pos, y_pos, 0));
					harris_value[2] += (coeff * dxImage.Pixel(x_pos, y_pos, 0) * dyImage.Pixel(x_pos, y_pos, 0));
					harris_value[3] += (coeff * dyImage.Pixel(x_pos, y_pos, 0) * dyImage.Pixel(x_pos, y_pos, 0));
				}
			}
			if (10 < x && x < w - 10 && 10 < y && y < h - 10)
				harrisImage.Pixel(x, y, 0) = (harris_value[0] * harris_value[3] - harris_value[1] * harris_value[2]) - 
										0.04 * (harris_value[0] + harris_value[3]) * (harris_value[0] + harris_value[3]);
			else
				harrisImage.Pixel(x, y, 0) = 0.0;
			harrisImage.Pixel(x, y, 0) *= MULTI_FACTOR;
        }
    }
}



// TO DO---------------------------------------------------------------------
// Loop through the harrisImage to threshold and compute the local maxima in a neighborhood
// srcImage:  image with Harris values
// destImage: Assign 1 to a pixel if it is above a threshold and is the local maximum in 3x3 window, 0 otherwise.
//    You'll need to find a good threshold to use.
void computeLocalMaxima(CFloatImage &srcImage,CByteImage &destImage)
{

	// MODIFY: find local maxima 5 x 5

	int w = srcImage.Shape().width;
    int h = srcImage.Shape().height;

	int sigma = 5;

	for (int y = 0; y < h; y++) {
		for (int x = 0; x < w; x++) {
			bool is_local_maxima = true;
			for (int y_offset = -sigma / 2; y_offset <= sigma / 2; y_offset++) {
				for (int x_offset = -sigma / 2; x_offset <= sigma / 2; x_offset++) {
					if (0 <= y + y_offset && y + y_offset < h && 0 <= x + x_offset && x + x_offset < w)
						if ((x_offset || y_offset) && srcImage.Pixel(x, y, 0) <= srcImage.Pixel(x + x_offset, y + y_offset, 0))
							is_local_maxima = false;
				}
			}
			if (is_local_maxima)
				destImage.Pixel(x, y, 0) = 1;
			else
				destImage.Pixel(x, y, 0) = 0;
		}
	}	
}


// Perform simple feature matching.  This just uses the SSD
// distance between two feature vectors, and matches a feature in the
// first image with the closest feature in the second image.  It can
// match multiple features in the first image to the same feature in
// the second image.
void ssdMatchFeatures(const FeatureSet &f1, const FeatureSet &f2, vector<FeatureMatch> &matches, double &totalScore) {
	int m = f1.size();
	int n = f2.size();

	printf("features %d, %d\n", m, n);

	matches.resize(m);
	totalScore = 0;

	double d;
	double dBest;
	int idBest;

	for (int i=0; i<m; i++) {
		dBest = 1e100;
		idBest = 0;

		for (int j=0; j<n; j++) {
			d = distanceSSD(f1[i].data, f2[j].data);

			if (d < dBest) {
				dBest = d;
				idBest = f2[j].id;
			}
		}

        matches[i].id1 = f1[i].id;
		matches[i].id2 = idBest;
		matches[i].score = dBest;
		totalScore += matches[i].score;
	}
}

// TODO: Write this function to perform ratio feature matching.  
// This just uses the ratio of the SSD distance of the two best matches as the score
// and matches a feature in the first image with the closest feature in the second image.
// It can match multiple features in the first image to the same feature in
// the second image.  (See class notes for more information, and the sshMatchFeatures function above as a reference)
void ratioMatchFeatures(const FeatureSet &f1, const FeatureSet &f2, vector<FeatureMatch> &matches, double &totalScore) 
{
	int m = f1.size();
	int n = f2.size();

	printf("features %d, %d\n", m, n);
	matches.clear();
	totalScore = 0;

	double d;
	vector < pair < double, int > > dis(n);

	for (int i=0; i<m; i++) {
		for (int j=0; j<n; j++) {
			d = distanceSSD(f1[i].data, f2[j].data);
			dis[j] = make_pair(d, f2[j].id);
		}
		
		sort(dis.begin(), dis.end());

		if (n > 2) {
			if (dis[1].first == 0) {
				printf("divide by 0.\n");
				continue;
			}
			if (dis[0].first / dis[1].first < 0.8) {
				FeatureMatch fm;
				fm.id1 = f1[i].id;
				fm.id2 = dis[0].second;
				fm.score = dis[0].first;
				totalScore += fm.score;
				matches.push_back(fm);
			}
		}
	}
}


// Convert Fl_Image to CFloatImage.
bool convertImage(const Fl_Image *image, CFloatImage &convertedImage) {
	if (image == NULL) {
		return false;
	}

	// Let's not handle indexed color images.
	if (image->count() != 1) {
		return false;
	}

	int w = image->w();
	int h = image->h();
	int d = image->d();

	// Get the image data.
	const char *const *data = image->data();

	int index = 0;

	for (int y=0; y<h; y++) {
		for (int x=0; x<w; x++) {
			if (d < 3) {
				// If there are fewer than 3 channels, just use the
				// first one for all colors.
				convertedImage.Pixel(x,y,0) = ((uchar) data[0][index]) / 255.0f;
				convertedImage.Pixel(x,y,1) = ((uchar) data[0][index]) / 255.0f;
				convertedImage.Pixel(x,y,2) = ((uchar) data[0][index]) / 255.0f;
			}
			else {
				// Otherwise, use the first 3.
				convertedImage.Pixel(x,y,0) = ((uchar) data[0][index]) / 255.0f;
				convertedImage.Pixel(x,y,1) = ((uchar) data[0][index+1]) / 255.0f;
				convertedImage.Pixel(x,y,2) = ((uchar) data[0][index+2]) / 255.0f;
			}

			index += d;
		}
	}
	
	return true;
}

// Convert CFloatImage to CByteImage.
void convertToByteImage(CFloatImage &floatImage, CByteImage &byteImage) {
	CShape sh = floatImage.Shape();

    assert(floatImage.Shape().nBands == byteImage.Shape().nBands);
	for (int y=0; y<sh.height; y++) {
		for (int x=0; x<sh.width; x++) {
			for (int c=0; c<sh.nBands; c++) {
				float value = floor(255*floatImage.Pixel(x,y,c) + 0.5f);

				if (value < byteImage.MinVal()) {
					value = byteImage.MinVal();
				}
				else if (value > byteImage.MaxVal()) {
					value = byteImage.MaxVal();
				}

				// We have to flip the image and reverse the color
				// channels to get it to come out right.  How silly!
				byteImage.Pixel(x,sh.height-y-1,sh.nBands-c-1) = (uchar) value;
			}
		}
	}
}

// Compute SSD distance between two vectors.
double distanceSSD(const vector<double> &v1, const vector<double> &v2) {
	int m = v1.size();
	int n = v2.size();

	if (m != n) {
		// Here's a big number.
		return 1e100;
	}

	double dist = 0;

	
	for (int i=0; i<m; i++) {
		dist += pow(v1[i]-v2[i], 2);
	}
	
	
	return sqrt(dist);
}

// Transform point by homography.
void applyHomography(double x, double y, double &xNew, double &yNew, double h[9]) {
	double d = h[6]*x + h[7]*y + h[8];

	xNew = (h[0]*x + h[1]*y + h[2]) / d;
	yNew = (h[3]*x + h[4]*y + h[5]) / d;
}

// Compute AUC given a ROC curve
double computeAUC(vector<ROCPoint> &results)
{
	double auc=0;
	double xdiff,ydiff;
	for (int i = 1; i < (int) results.size(); i++)
    {
        //fprintf(stream,"%lf\t%lf\t%lf\n",thresholdList[i],results[i].falseRate,results[i].trueRate);
		xdiff=(results[i].falseRate-results[i-1].falseRate);
		ydiff=(results[i].trueRate-results[i-1].trueRate);
		auc=auc+xdiff*results[i-1].trueRate+xdiff*ydiff/2;
    	    
    }
	return auc;
}